import Joi from 'joi';
import { AccessInfo } from '../auth';
export declare enum RecordStatus {
    ACTIVE = "ACTIVE",
    INACTIVE = "INACTIVE"
}
export declare type RoleModel = {
    select?: Record<string, boolean | RoleModelSelect>;
    findUnique?: RoleModelFindUnique;
    findMany?: RoleModelFindMany;
    update?: RoleModelUpdate;
    updateMany?: RoleModelUpdateMany;
    delete?: RoleModelDelete;
    deleteMany?: RoleModelDeleteMany;
    create?: RoleModelCreate;
    createMany?: RoleModelCreateMany;
};
export declare type RoleModelSelect = {
    model?: string;
    oneToOne?: boolean;
};
export declare type RoleModelFindUnique<T = any> = {
    where?: Record<string, T>;
};
export declare type RoleModelFindMany<T = any> = {
    where?: Record<string, T>[];
};
export declare type RoleModelUpdate<T = any> = {
    where?: Record<string, T>;
};
export declare type RoleModelUpdateMany<T = any> = {
    where?: Record<string, T>[];
};
export declare type RoleModelDelete<T = any> = {
    where?: Record<string, T>;
};
export declare type RoleModelDeleteMany<T = any> = {
    where?: Record<string, T>[];
};
export declare type RoleModelCreate<T = any> = {
    data?: Record<string, T>;
};
export declare type RoleModelCreateMany<T = any> = {
    data?: Record<string, T>;
};
export declare type BaseModelErrorHandler = (error: TypeError, params: BaseModelMiddlewareParams, context: BaseModelQueryContext) => never;
export declare type BaseModelQueryHandler<T> = () => Promise<T>;
export declare const OmitCreateInput: {
    createdBy: number;
    createdByName: string;
};
export declare const OmitUpdateInput: {
    updatedBy: number;
    updatedByName: string;
};
export declare type BaseModelMiddlewareParams = {
    model?: string;
    action: string;
    args: any;
    dataPath: string[];
    runInTransaction: boolean;
};
export declare type BaseModelBatchPayload = {
    count: number;
};
export declare type BaseModelSchemas = {
    schemaCreateInput?: Joi.ObjectSchema;
    schemaUpdateInput?: Joi.ObjectSchema;
};
export declare type BaseModelQueryContext = {
    schemas: BaseModelSchemas;
    accessInfo: AccessInfo;
    disableRestriction?: boolean;
};
export declare class BaseModelMiddleware<PrismaClient> {
    readonly client: PrismaClient;
    private errorHandler;
    private asyncLocalStorage;
    constructor(client: PrismaClient, errorHandler?: BaseModelErrorHandler);
    run<T>(context: BaseModelQueryContext, promise: BaseModelQueryHandler<T>): Promise<T>;
    private process;
    private packFindUniqueArgs;
    private packFindManyArgs;
    private packCreateArgs;
    private packCreateManyArgs;
    private packUpdateArgs;
    private packUpdateManyArgs;
    private packUpsertArgs;
    private packDeleteArgs;
    private packDeleteManyArgs;
    private restrictCreate;
    private restrictSelect;
    private restrictInclude;
    private restrictWhere;
    private restrictWhereUnique;
    private restrictFields;
    private getRoleModel;
    private selectableModel;
}
